@@ -43,14 +43,14 @@ def _classes(self):
         self.attr.__class__.__name__.lower(),
         ]
     if self.widget is not None:
-        classes.append(self.widget.__class__.__name__.lower())
+        classes.append(self.widget.widget.__class__.__name__.lower())
     else:
         classes.append('defaultwidget')
     if self.required:
         classes.append('required')
     if self.widget is not None and self.widget.css_class:
         classes.append(self.widget.css_class)
-    if self.error:
+    if self.error.val:
         classes.append('error')
     return ' '.join(classes)
             
@@ -68,6 +68,38 @@ def starify(name):
     name = '.'.join(newname)
     return name
 
+
+def fall_back_renderer(renderer, name, widget, vars):
+    import mako
+    try:
+        return renderer('/formish/widgets/%s/%s.html'%(widget,name), vars)
+    except mako.exceptions.TopLevelLookupException, e:
+        return renderer('/formish/%s.html'%(name), vars)
+    
+
+class TemplatedString(object):
+    """
+    A callable, teplated string
+    """
+    def __init__(self, obj, attr_name, val):
+        self.obj = obj
+        self.attr_name = attr_name
+        self.val = val
+
+    def __str__(self):
+        if not self.val:
+            return ''
+        return self.val
+
+    def __call__(self):
+        renderer = self.obj.form.renderer
+        name = '%s_%s'%(self.obj.type,self.attr_name)
+        widget = self.obj.widget._template
+        vars = {'field':self.obj}
+        return fall_back_renderer(renderer, name, widget, vars)
+
+
+
 class Field(object):
     """
     A wrapper for a schema field type that includes form information.
@@ -108,9 +140,10 @@ class Field(object):
     def description(self):
         """ The Field schema's description """
         try:
-            return self.form.get_item_data(self.name,'description')
+            val =  self.form.get_item_data(self.name,'description')
         except KeyError:
-            return self.attr.description        
+            val = self.attr.description        
+        return TemplatedString(self, 'description', val)
 
 
     @property
@@ -142,7 +175,12 @@ class Field(object):
     @property
     def error(self):
         """ Lazily get the error from the form.errors when needed """
-        return self.form.errors.get(self.name, None)
+        error = self.form.errors.get(self.name, None)
+        if error is not None:
+            val = str(error)
+        else: 
+            val = ''
+        return TemplatedString(self, 'error', val)
 
 
     @property
@@ -153,14 +191,54 @@ class Field(object):
             widget_type = self.form.get_item_data(starify(self.name),'widget')
         except KeyError:
             widget_type = widgets.Input()
-        return widget_type
+        return BoundWidget(widget_type, self)
+
         
 
     def __call__(self):
         """ returns a serialisation for this field using the form's renderer """
-        return self.form.renderer('/formish/Field.html', {'f':self})
+        renderer = self.form.renderer
+        name = 'field'
+        widget = self.widget._template
+        vars = {'field':self}
+        return fall_back_renderer(renderer, name, widget, vars)
             
+    def label(self):
+        """ returns the templated title """
+        renderer = self.form.renderer
+        name = 'field_label'
+        widget = self.widget._template
+        vars = {'field':self}
+        return fall_back_renderer(renderer, name, widget, vars)
     
+    def inputs(self):
+        """ returns the templated widget """
+        renderer = self.form.renderer
+        name = 'field_inputs'
+        widget = self.widget._template
+        vars = {'field':self}
+        return fall_back_renderer(renderer, name, widget, vars)
+
+
+class CollectionFieldsWrapper(object):
+    """
+    Allow fields attr of a form to be accessed (as a generator) but also callable
+    """
+    def __init__(self, collection):
+        self.collection = collection
+        self.val= collection.collection_fields()
+
+    def __iter__(self):
+        if self.val is None:
+            return iter([])
+        return iter(self.val)
+
+    def __call__(self):
+        renderer = self.collection.form.renderer
+        name = '%s_fields'%self.collection.type
+        widget = self.collection.widget._template
+        vars = {'field':self.collection}
+        return fall_back_renderer(renderer, name, widget, vars)
 
 class Collection(object):
     """
@@ -194,7 +272,8 @@ class Collection(object):
     @property
     def description(self):
         """ Returns the schema's description """
-        return self.attr.description        
+        val = self.attr.description        
+        return TemplatedString(self,'description',val)
 
 
     @property
@@ -232,7 +311,8 @@ class Collection(object):
     @property
     def error(self):
         """ Lazily get the error from the form.errors when needed """
-        return self.form.errors.get(self.name, None)
+        val = self.form.errors.get(self.name, None)
+        return TemplatedString(self, 'error', val)
 
 
     @property
@@ -240,9 +320,9 @@ class Collection(object):
         """ return the fields widget bound with extra params. """
         
         try:
-            widget_type = self.form.get_item_data(starify(self.name),'widget')
+            widget_type = BoundWidget(self.form.get_item_data(starify(self.name),'widget'),self)
         except KeyError:
-            widget_type = widgets.SequenceDefault()
+            widget_type = BoundWidget(widgets.SequenceDefault(),self)
         return widget_type
 
 
@@ -265,6 +345,10 @@ class Collection(object):
         """ The schemaish attrs below this collection """
         return self.attr.attrs
 
+    def collection_fields(self):
+        for attr in self.attrs:
+            yield self.bind(attr[0], attr[1])        
+        
 
     @property
     def fields(self):
@@ -272,13 +356,12 @@ class Collection(object):
         Iterate through the fields, lazily bind the schema to the fields
         before returning.
         """
-        for attr in self.attr.attrs:
-            yield self.bind(attr[0], attr[1])        
+        return CollectionFieldsWrapper(self)
 
 
     def bind(self, attr_name, attr):
         """ 
-        return cached schema as a field; Otherwise bind the attr to a
+        return cached bound schema as a field; Otherwise bind the attr to a
         Group or Field as appropriate and store on the _fields cache
         
         :param attr_name:     Form Field/Group identifier
@@ -295,24 +378,52 @@ class Collection(object):
                 keyprefix = '%s.%s'% (self.name, attr_name)
                 
             if isinstance(attr, schemaish.Sequence):
-                field = Sequence(keyprefix, attr, self.form)
+                bound_field = Sequence(keyprefix, attr, self.form)
             elif isinstance(attr, schemaish.Structure):
-                field = Group(keyprefix, attr, self.form)
+                bound_field = Group(keyprefix, attr, self.form)
             else:
-                field = Field(keyprefix, attr, self.form)
-            self._fields[attr_name] = field
-            return field
+                bound_field = Field(keyprefix, attr, self.form)
+            self._fields[attr_name] = bound_field
+            return bound_field
 
 
-    def __call__(self):
-        return self.form.renderer('/formish/Field.html', {'f':self})
-
 
     def __repr__(self):
         return '<formish %s name="%s">'% (self.type, self.name)
-   
 
 
+    def __call__(self):
+        """ returns a serialisation for this field using the form's renderer """
+        renderer = self.form.renderer
+        name = self.type
+        widget = self.widget._template
+        vars = {'field':self}
+        return fall_back_renderer(renderer, name, widget, vars)
+            
+    def label(self):
+        """ returns the templated title """
+        renderer = self.form.renderer
+        name = '%s_label'%self.type
+        widget = self.widget._template
+        vars = {'field':self}
+        return fall_back_renderer(renderer, name, widget, vars)
+         
+    def metadata(self):
+        """ returns the metadata """
+        renderer = self.form.renderer
+        name = '%s_metadata'%self.type
+        widget = self.widget._template
+        vars = {'field':self}
+        return fall_back_renderer(renderer, name, widget, vars)
+
+    def inputs(self):
+        """ returns the templated widget """
+        renderer = self.form.renderer
+        name = '%s_inputs'%self.type
+        widget = self.widget._template
+        vars = {'field':self}
+        return fall_back_renderer(renderer, name, widget, vars)
+
 class Group(Collection):
     """
     A group is a basic collection with a different template
@@ -329,8 +440,7 @@ class Sequence(Collection):
     type = 'sequence'
     _template = 'ssequence'
 
-    @property
-    def fields(self):
+    def collection_fields(self):
         """ 
         For sequences we check to see if the name is numeric. As names cannot be numeric normally, the first iteration loops 
         on a fields values and spits out a 
@@ -371,7 +481,49 @@ class Sequence(Collection):
         return self.bind('*', self.attr.attr)
             
 
+class BoundWidget(object):
+    """
+    Because widget's need to be able to render themselves
+    """   
+    
 
+    def __init__(self, widget, field):
+        self.__dict__['widget'] = widget
+        self.__dict__['field'] = field
+     
+
+    def __getattr__(self, name):
+        return getattr(self.widget, name)
+
+    def __setattr__(self, name, value):
+        setattr(self.widget, name, value)
+
+    def __call__(self):
+        return self.field.form.renderer('/formish/widgets/%s/widget.html'%self._template, {'field':self.field})
+
+    def __repr__(self):
+        attrclassstr = str(self.field.attr.__class__)
+        if attrclassstr[8:22] == 'schemaish.attr':
+            attrname = attrclassstr[23:-2]
+        else:
+            attrname = attrclassstr[8:-2]
+        return '<bound widget name="%s", widget="%s", type="%s">'%(self.field.name, self.widget._template, attrname)
+
+class FormFieldsWrapper(object):
+    """
+    Allow fields attr of a form to be accessed (as a generator) but also callable
+    """
+    def __init__(self, form):
+        self.form = form
+        self.val= form.structure.fields
+
+    def __iter__(self):
+        if self.val is None:
+            return iter([])
+        return iter(self.val)
+
+    def __call__(self):
+        return self.form.renderer('/formish/form_fields.html', {'form':self.form})
     
 
 class Form(object):
@@ -432,7 +584,7 @@ class Form(object):
         self.defaults = defaults
         self.errors = dottedDict(errors)
         self.error = None
-        self.actions = []
+        self._actions = []
         self.action_url = action_url
         if renderer is not None:
             self.renderer = renderer
@@ -481,9 +633,9 @@ class Form(object):
         :type label: string
         
         """
-        if name in [action.name for action in self.actions]:
+        if name in [action.name for action in self._actions]:
             raise ValueError('Action with name %r already exists.'% name)
-        self.actions.append( Action(callback, name, label) )              
+        self._actions.append( Action(callback, name, label) )              
 
     def action(self, request, *args):
         """
@@ -495,12 +647,12 @@ class Form(object):
 
         :arg args: list of arguments Pass through to the callback
         """
-        if len(self.actions)==0:
+        if len(self._actions)==0:
             raise validation.NoActionError('The form does not have any actions')
-        for action in self.actions:
+        for action in self._actions:
             if action.name in request.POST.keys():
                 return action.callback(request, self, *args)
-        return self.actions[0].callback(request, self, *args)
+        return self._actions[0].callback(request, self, *args)
 
 
     def get_unvalidated_data(self, request_data, raise_exceptions=True):
@@ -648,6 +800,7 @@ class Form(object):
         return FormAccessor(self, key)
 
 
+
     @property
     def fields(self):
         """
@@ -655,7 +808,8 @@ class Form(object):
         the form (e.g. if a field is a subsection or sequence, it will be up to
         the application to iterate that field's fields.
         """
-        return self.structure.fields
+        return FormFieldsWrapper(self)
+
 
 
     def get_field(self, name):
@@ -677,10 +831,26 @@ class Form(object):
         """
         Calling the Form generates a serialisation using the form's renderer
         """
-        return self.renderer('/formish/Form.html', {'form':self})
+        return self.renderer('/formish/form.html', {'form':self})
+
+    def header(self):
+        """ Return just the header part of the template """
+        return self.renderer('/formish/form_header.html', {'form':self})
         
-    
-    
+    def footer(self):
+        """ Return just the footer part of the template """
+        return self.renderer('/formish/form_footer.html', {'form':self})
+        
+    def metadata(self):
+        """ Return just the metada part of the template """
+        return self.renderer('/formish/form_metadata.html', {'form':self})
+
+    def actions(self):
+        """ Return just the actions part of the template """
+        return self.renderer('/formish/form_actions.html', {'form':self})
+   
+
+
 class FormAccessor(object):
     """
     Helps in setting item_data on a form
@@ -709,5 +879,8 @@ class FormAccessor(object):
     
     def __getitem__(self, key):
         return FormAccessor(self.form, key, prefix=self.key)
+
+    def __call__(self):
+        return self.form.get_field(self.key)()
     
         
