@@ -8,7 +8,12 @@ log = logging.getLogger(__name__)
 # Make sure forms have the doc string with triple quotes 
 # on a separate line as this file is parsed
 
-def SimpleString():
+
+################
+#
+#  Simple Types
+
+def String():
     """
     A simple form with a single string field
     """
@@ -17,8 +22,8 @@ def SimpleString():
     form = formish.Form(schema, 'form')
     return form
 
-def functest_SimpleString(self, sel):
-    sel.open("/SimpleString")
+def functest_String(self, sel):
+    sel.open("/String")
 
     sel.click("form-action-submit")
     sel.wait_for_page_to_load("30000")
@@ -39,25 +44,25 @@ def functest_SimpleString(self, sel):
 
     return
 
-def unittest_SimpleString(self):
+def unittest_String(self):
     # Test no data
-    f = SimpleString()
+    f = String()
     self.assertIdHasValue(f, 'form-myStringField', '')
     # Test None data
-    f = SimpleString()
+    f = String()
     testdata = {'myStringField': None}
     f.defaults = testdata
     self.assertIdHasValue(f, 'form-myStringField', '')
     self.assertRoundTrip(f, testdata)
     # Test sample data
-    f = SimpleString()
+    f = String()
     testdata = {'myStringField': '8'}
     f.defaults = testdata
     self.assertIdHasValue(f, 'form-myStringField', '8')
     self.assertRoundTrip(f, testdata)
 
 
-def SimpleInteger():
+def Integer():
     """
     A simple form with a single integer field
     """
@@ -66,8 +71,8 @@ def SimpleInteger():
     form = formish.Form(schema, 'form')
     return form
 
-def functest_SimpleInteger(self,sel):
-    sel.open("/SimpleInteger")
+def functest_Integer(self,sel):
+    sel.open("/Integer")
 
     sel.click("form-action-submit")
     sel.wait_for_page_to_load("30000")
@@ -94,24 +99,24 @@ def functest_SimpleInteger(self,sel):
 
     return
 
-def unittest_SimpleInteger(self):
+def unittest_Integer(self):
     # Test no data
-    f = SimpleInteger()
+    f = Integer()
     self.assertIdHasValue(f, 'form-myIntegerField', '')
     # Test None data
-    f = SimpleInteger()
+    f = Integer()
     testdata = {'myIntegerField': None}
     f.defaults = testdata
     self.assertIdHasValue(f, 'form-myIntegerField', '')
     self.assertRoundTrip(f, testdata)
     # Test sample data
-    f = SimpleInteger()
+    f = Integer()
     testdata = {'myIntegerField': 8}
     f.defaults = testdata
     self.assertIdHasValue(f, 'form-myIntegerField', '8')
     self.assertRoundTrip(f, testdata)
 
-def SimpleDate():
+def Date():
     """
     A simple form with a single integer field
     """
@@ -120,9 +125,9 @@ def SimpleDate():
     form = formish.Form(schema, 'form')
     return form
 
-def functest_SimpleDate(self, sel):
+def functest_Date(self, sel):
 
-    sel.open("/SimpleDate")
+    sel.open("/Date")
 
     sel.click("form-action-submit")
     sel.wait_for_page_to_load("30000")
@@ -149,7 +154,7 @@ def functest_SimpleDate(self, sel):
 
     return
 
-def SimpleFloat():
+def Float():
     """
     A simple form with a single integer field
     """
@@ -158,8 +163,8 @@ def SimpleFloat():
     form = formish.Form(schema, 'form')
     return form
 
-def functest_SimpleFloat(self, sel):
-    sel.open("/SimpleFloat")
+def functest_Float(self, sel):
+    sel.open("/Float")
 
     sel.click("form-action-submit")
     sel.wait_for_page_to_load("30000")
@@ -188,7 +193,7 @@ def functest_SimpleFloat(self, sel):
 
 
 
-def SimpleBoolean():
+def Boolean():
     """
     A simple form with a single boolean field
     """
@@ -197,8 +202,8 @@ def SimpleBoolean():
     form = formish.Form(schema, 'form')
     return form
 
-def functest_SimpleBoolean(self, sel):
-    sel.open("/SimpleBoolean")
+def functest_Boolean(self, sel):
+    sel.open("/Boolean")
 
     sel.click("form-action-submit")
     sel.wait_for_page_to_load("30000")
@@ -232,7 +237,7 @@ def functest_SimpleBoolean(self, sel):
     return
 
 
-def SimpleDecimal():
+def Decimal():
     """
     A simple form with a single decimal field
     """
@@ -242,8 +247,8 @@ def SimpleDecimal():
     return form
 
 
-def functest_SimpleDecimal(self, sel):
-    sel.open("/SimpleDecimal")
+def functest_Decimal(self, sel):
+    sel.open("/Decimal")
 
     sel.click("form-action-submit")
     sel.wait_for_page_to_load("30000")
@@ -270,8 +275,6 @@ def functest_SimpleDecimal(self, sel):
 
     return
 
-
-
 def RequiredStringAndCheckbox():
     """
     Testing that a checkbox is working properly when another required field is missing
@@ -303,12 +306,10 @@ def functest_RequiredStringAndCheckbox(self, sel):
     sel.wait_for_page_to_load("30000")
     try: self.failUnless(sel.is_text_present("{'myBoolean': True, 'myString': u'anythingelse'}"))
     except AssertionError, e: self.verificationErrors.append(str(e))
-    
 
     return
 
-
-def SimpleFile():
+def File():
     """
     A simple form with a single integer field
     """
@@ -318,8 +319,8 @@ def SimpleFile():
     form['myFileField'].widget = formish.FileUpload(filehandler=TempFileHandlerWeb())
     return form
 
-def functest_SimpleFile(self, sel):
-    sel.open("/SimpleFile")
+def functest_File(self, sel):
+    sel.open("/File")
 
     sel.click("form-action-submit")
     sel.wait_for_page_to_load("30000")
@@ -336,42 +337,351 @@ def functest_SimpleFile(self, sel):
 
     return
 
+#########################
+#
+#   String Widgets
+#
 
-
-def StringWidgets():
+def Input():
     """
-    A demonstration of simple string type widgets
-
-    * **Input** fields are the default and need not be specified but you might want to specify a 'strip' argument
-    * **TextArea** take cols and rows keyword arguments (css usually overrides this). it can also take a 'strip' argument
+    Simple input field with a strip parameter
     """
     schema = schemaish.Structure()
-    schema.add('myInputStrip', schemaish.String())
-    schema.add('myTextArea', schemaish.String())
-    schema.add('myTextAreaCustom', schemaish.String())
-    schema.add('myTextAreaStrip', schemaish.String())
+    schema.add('inputStrip', schemaish.String())
 
     form = formish.Form(schema, 'form')
-    form['myInputStrip'].widget = formish.Input(strip=True)
-    form['myTextArea'].widget = formish.TextArea()
-    form['myTextAreaCustom'].widget = formish.TextArea(cols=20,rows=4)
-    form['myTextAreaStrip'].widget = formish.TextArea(strip=True)
+    form['inputStrip'].widget = formish.Input(strip=True)
     return form
 
-def SelectWidgets():
+def Hidden():
     """
-    A set of widget demonstrations using choices of various kinds
+    Hidden Field with a visible friend.. 
+    """
+    schema = schemaish.Structure()
+    schema.add('Visible', schemaish.String())
+    schema.add('Hidden', schemaish.String())
+
+    form = formish.Form(schema, 'form')
+    form['Hidden'].widget = formish.Hidden()
+    return form
+
+def Password():
+    """
+    Using a password html element
+    """
+    schema = schemaish.Structure()
+    schema.add('Password', schemaish.String())
+
+    form = formish.Form(schema, 'form')
+    form['Password'].widget = formish.Password()
+    return form
+
+def CheckedPassword():
+    """
+    Simple input field with a strip parameter
+    """
+    schema = schemaish.Structure()
+    schema.add('CheckedPassword', schemaish.String())
+
+    form = formish.Form(schema, 'form')
+    form['CheckedPassword'].widget = formish.CheckedPassword()
+    return form
+
+def TextAreaSimple():
+    """
+    Simple text area
+    """
+    schema = schemaish.Structure()
+    schema.add('textArea', schemaish.String())
+
+    form = formish.Form(schema, 'form')
+    form['textArea'].widget = formish.TextArea()
+    return form
+
+def TextAreaColsAndRows():
+    """
+    Passing cols and rows to a text area
+    """
+    schema = schemaish.Structure()
+    schema.add('textAreaCustom', schemaish.String())
+
+    form = formish.Form(schema, 'form')
+    form['textAreaCustom'].widget = formish.TextArea(cols=20,rows=4)
+    return form
+
+def TextAreaStrip():
+    """
+    Stripping text area input
+    """
+    schema = schemaish.Structure()
+    schema.add('textAreaStrip', schemaish.String())
+
+    form = formish.Form(schema, 'form')
+    form['textAreaStrip'].widget = formish.TextArea(strip=True)
+    return form
+
+#######################
+#
+#   Validation
+
+def Required():
+    """
+    Required Fields
+    """
+    schema = schemaish.Structure()
+    schema.add('required', schemaish.String(validator=validatish.Required()))
+
+    form = formish.Form(schema, 'form')
+    return form
+
+def MinLength():
+    """
+    Minimum Length fields - this one is min length four chars
+    """
+    schema = schemaish.Structure()
+    schema.add('min', schemaish.String(validator=validatish.Length(min=4)))
+
+    form = formish.Form(schema, 'form')
+    return form
+
+def MaxLength():
+    """
+    Maximum Length fields - this one is max length eight chars
+    """
+    schema = schemaish.Structure()
+    schema.add('max', schemaish.String(validator=validatish.Length(max=8)))
+
+    form = formish.Form(schema, 'form')
+    return form
+
+def MinMaxLength():
+    """
+    Minimum and maximum length
+    """
+    schema = schemaish.Structure()
+    schema.add('minmax', schemaish.String(validator=validatish.Length(min=4,max=8)))
+
+    form = formish.Form(schema, 'form')
+    return form
+
+def MinLengthCheckboxMultiChoice():
+    """
+    A checkbox multi choice with minimum length 3 (NOT WORKING)
+    """
+    schema = schemaish.Structure()
+    schema.add('multiChoice', schemaish.Sequence(schemaish.String(validation=validatish.Length(min=2))))
+    options = [(1,'a'),(2,'b'),(3,'c')]
+
+    form = formish.Form(schema, 'form')
+    form['multiChoice'].widget = formish.CheckboxMultiChoice(options)
+    return form
+
+def PlainText():
+    """
+    Plain Text (alphanum only)
+    """
+    schema = schemaish.Structure()
+    schema.add('plainText', schemaish.String(validator=validatish.PlainText()))
+
+    form = formish.Form(schema, 'form')
+    return form
+
+def OneOf():
+    """
+    One of a set of
+    """
+    schema = schemaish.Structure()
+    items = ['one','two','three']
+    schema.add('oneOf', schemaish.String(validator=validatish.OneOf(items)))
+
+    form = formish.Form(schema, 'form')
+    return form
+
+def All():
+    """
+    Required and Plain Text (alphanum only plus _ and -)
+    """
+    schema = schemaish.Structure()
+    schema.add('requiredPlainText', schemaish.String(validator=validatish.All(validatish.Required(),validatish.PlainText(extra='-_'))))
+
+    form = formish.Form(schema, 'form')
+    return form
+
+########################
+#
+#   Checkbox
+
+def Checkbox():
+    """
+    Simple Boolean Checkbox
+    """
+    schema = schemaish.Structure()
+    schema.add('checkbox', schemaish.Boolean())
+
+    form = formish.Form(schema, 'form')
+    form['checkbox'].widget = formish.Checkbox()
+    return form
+
+
+#########################
+#
+#   Select Widgets
+#
+
+def SelectChoice():
+    """
+    A basic select choice
     """
     schema = schemaish.Structure()
     schema.add('mySelect', schemaish.String())
-    options = [('a',1),('b',2),('c',3)]
+    options = [(1,'a'),(2,'b'),(3,'c')]
 
     form = formish.Form(schema, 'form')
     form['mySelect'].widget = formish.SelectChoice(options)
     return form
 
 
-def SequenceOfStructures():
+def SelectChoiceNoneOption():
+    """
+    Setting a None Option on the select choice element
+    """
+    schema = schemaish.Structure()
+    schema.add('mySelect', schemaish.String())
+    options = [(1,'a'),(2,'b'),(3,'c')]
+
+    form = formish.Form(schema, 'form')
+    form['mySelect'].widget = formish.SelectChoice(options,none_option=(None, '--select--'))
+    return form
+
+def SelectChoiceCallableOptions():
+    """
+    Passing in a callable list of options
+    """
+    schema = schemaish.Structure()
+    schema.add('mySelect', schemaish.String())
+    def _():
+        options = [(1,'a'),(2,'b'),(3,'c')]
+        for option in options:
+            yield option
+
+    form = formish.Form(schema, 'form')
+    form['mySelect'].widget = formish.SelectChoice(_)
+    return form
+
+def RadioChoice():
+    """
+    A basic select choice
+    """
+    schema = schemaish.Structure()
+    schema.add('myRadio', schemaish.String())
+    options = [(1,'a'),(2,'b'),(3,'c')]
+
+    form = formish.Form(schema, 'form')
+    form['myRadio'].widget = formish.RadioChoice(options)
+    return form
+
+
+def RadioChoiceNoneOption():
+    """
+    Setting a None Option on the select choice element
+    """
+    schema = schemaish.Structure()
+    schema.add('myRadio', schemaish.String())
+    options = [(1,'a'),(2,'b'),(3,'c')]
+
+    form = formish.Form(schema, 'form')
+    form['myRadio'].widget = formish.RadioChoice(options,none_option=(None, '--select--'))
+    return form
+
+def RadioChoiceCallableOptions():
+    """
+    Passing in a callable list of options
+    """
+    schema = schemaish.Structure()
+    schema.add('myRadio', schemaish.String())
+    def _():
+        options = [(1,'a'),(2,'b'),(3,'c')]
+        for option in options:
+            yield option
+
+    form = formish.Form(schema, 'form')
+    form['myRadio'].widget = formish.RadioChoice(_)
+    return form
+
+#########################
+#
+#   Multi Select Widgets
+#
+
+def CheckboxMultiChoice():
+    """
+    A checkbox representing a set of values
+    """
+    schema = schemaish.Structure()
+    schema.add('multiChoice', schemaish.Sequence(schemaish.String()))
+    options = [(1,'a'),(2,'b'),(3,'c')]
+
+    form = formish.Form(schema, 'form')
+    form['multiChoice'].widget = formish.CheckboxMultiChoice(options)
+    return form
+
+
+def CheckboxMultiChoiceTree():
+    """
+    A checkbox representing a set of values
+    """
+    schema = schemaish.Structure()
+    schema.add('multiChoice', schemaish.Sequence(schemaish.String()))
+    options = [('a','Top Level A'),('a.x','Second Level X'),('a.y','Second Level Y'),('b','First Level B')]
+
+    form = formish.Form(schema, 'form')
+    form['multiChoice'].widget = formish.CheckboxMultiChoiceTree(options)
+    return form
+
+#########################
+#
+#   Structures
+#
+
+def SimpleStructure():
+    """
+    A simple structure
+    """
+    structure = schemaish.Structure()
+    structure.add( 'a', schemaish.String() )
+    structure.add( 'b', schemaish.Integer() )
+
+    schema = schemaish.Structure()
+    schema.add( 'myStruct', structure )
+
+    form = formish.Form(schema, 'form')
+    return form
+  
+def UploadStructure():
+    """
+    A structure with a file upload
+    """
+    structure = schemaish.Structure()
+    structure.add( 'a', schemaish.File() )
+    structure.add( 'b', schemaish.Integer() )
+
+    schema = schemaish.Structure()
+    schema.add( 'myStruct', structure)
+
+    form = formish.Form(schema, 'form')
+
+    form['myStruct.a'].widget = formish.FileUpload(filehandler=TempFileHandlerWeb())
+    return form
+
+
+
+#########################
+#
+#   Sequences
+#
+
+
+def SequenceOfSimpleStructures():
     """
     A structure witin a sequence, should be enhanced with javascript
     """
@@ -417,8 +727,6 @@ def functest_SequenceOfUploadStructures(self, sel):
 
     return
 
-
-    
 def SequenceOfStructuresWithSelects():
     """
     A sequence including selects
@@ -456,3 +764,145 @@ def functest_SequenceOfStructuresWithSelects(self, sel):
 
     return
 
+
+
+def GranularFormLayout():
+    """
+    A simple demonstration of partial rendering of parts of forms.
+    """
+
+    schema = schemaish.Structure()
+    schema.add( 'firstName', schemaish.String(title='First Name',description='The name before your last one',validator=validatish.Required()) )
+
+    form = formish.Form(schema, 'form')
+
+    return form
+
+
+def template_GranularFormLayout():
+    """
+
+${form()|n}
+
+<h2>Form</h2>
+
+<h3>Normal Form</h3>
+<pre>
+${form()}
+</pre>
+
+<h4>Form Header, metadata, actions and Footer</h4>
+<pre>
+
+${form.header()}
+<hr />
+${form.metadata()}
+<hr />
+${form.actions()}
+<hr />
+${form.footer()}
+</pre>
+
+<h4>First Name</h4>
+<pre>
+${form['firstName']()}
+</pre>
+
+<h3>First Name (.title)</h4>
+<pre>
+${form['firstName'].title}
+</pre>
+
+<h3>First Name (.label())</h4>
+<pre>
+${form['firstName'].label()}
+</pre>
+
+<h3>First Name (.inputs())</h4>
+<pre>
+${form['firstName'].inputs()}
+</pre>
+
+<h3>First Name (.widget())</h4>
+<pre>
+${form['firstName'].widget()}
+</pre>
+
+<h3>First Name (.error())</h4>
+<pre>
+${form['firstName'].error()}
+</pre>
+
+<h3>First Name (.error)</h4>
+<pre>
+${form['firstName'].error}
+</pre>
+
+<h3>First Name (.cssname)</h4>
+<pre>
+${form['firstName'].cssname}
+</pre>
+
+<h3>First Name (.classes)</h4>
+<pre>
+${form['firstName'].classes}
+</pre>
+
+<h3>First Name (.value)</h4>
+<pre>
+${form['firstName'].value}
+</pre>
+
+<h3>First Name (.required)</h4>
+<pre>
+${form['firstName'].required}
+</pre>
+
+
+
+<h3>First Name (.description)</h4>
+<pre>
+${form['firstName'].description}
+</pre>
+
+<h3>First Name (.description())</h4>
+<pre>
+${form['firstName'].description()}
+</pre>
+    """
+
+    
+def CustomisedFormLayout():
+    """
+    A custom form
+    """
+    schema = schemaish.Structure()
+    schema.add( 'firstName', schemaish.String())
+    schema.add( 'surname', schemaish.String(description='THIS MUST BE YOUR SURNAME') )
+
+    form = formish.Form(schema, 'form')
+
+    form['surname'].widget = formish.Input(css_class="surnamewidget")
+
+    return form
+
+
+def template_CustomisedFormLayout():
+    """
+${form.header()|n}
+${form.metadata()|n}
+
+${form['firstName']()|n}
+
+<div id="${form['surname'].cssname}-field" class="${form['surname'].classes}">
+  <strong>${form['surname'].description}</strong>
+  <em>${form['surname'].error}</em>
+  ${form['surname'].widget()|n}
+</div>
+
+${form.actions()|n}
+${form.footer()|n}
+
+    """
+
+    
